﻿/*
 * Copyright (c) 2010, Koji Tsukada<tsuka@mobiquitous.com> 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Collections.Generic;
using System.Text;

namespace MobiServer.Devices.Xbee
{
    public class DataDevice
    {
        public const byte START_BYTE = 0x7E;

        long serialNumber;
        short networkAddress;
        bool[] digitalEnables;
        bool[] analogEnables;
        bool[] digitalData;
        short[] analogData;

        public DataDevice(long serialNumber, short networkAddress, bool[] digitalEnables, bool []analogEnables, bool[] digitalData, short[] analogData)
        {
            this.serialNumber = serialNumber;
            this.networkAddress = networkAddress;
            this.digitalEnables = digitalEnables;
            this.analogEnables = analogEnables;
            this.digitalData = digitalData;
            this.analogData = analogData;
        }

        public long SerialNumber{
            get { return this.serialNumber; }
        }

        public short NetworkAddress
        {
            get { return this.networkAddress; }
        }

        public bool[] DigitalEnables
        {
            get { return this.digitalEnables; }
        }

        public bool[] AnalogEnables
        {
            get { return this.analogEnables; }
        }

        public bool[] DigitalData
        {
            get { return this.digitalData; }

        }

        public short[] AnalogData
        {
            get { return this.analogData; }

        }

        static int CountFlags(bool[] flags)
        {
            int count = 0;

            foreach (bool b in flags)
            {
                if (b)
                    count += 1;
            }

            return count;
        }

        static bool[] BytesToBool(byte[] bytes)
        {
            //const int BYTE_LENGTH = 8;
            bool [] bools = new bool[bytes.Length * 8];
            int cnt = 0;

            for (int i = 0; i< bytes.Length; i++){
                for (int j = 0; j < 8; j++)
                {
                    cnt = i * 8 + j;
                    if ((bytes[i] & (0x01 << j)) != 0)
                    {                      
                        bools[cnt] = true;
                    }
                    else
                    {
                        bools[cnt] = false;
                    }
                }
            }
            return bools;
        }

        static bool CalcCheckSum(byte[] bytes)
        {
            int sum = 0;

            foreach (byte b in bytes)
            {
                sum += b;
            }
            if ((sum & 0xFF) == 0xFF)
            {
                return true;
            }
            else
            {
                return false;
            }

        }

        public static DataDevice Parse(byte[] bytes)
        {
            DataDevice device = null;

            int digitalCnt;
            int analogCnt;

            byte command;
            long serialNumber;
            short networkAddress;
            bool[] digitalEnables;  
            bool[] analogEnables;
            bool[] digitalData;
            short[] analogData;

            try
            {
                if(!CalcCheckSum(bytes)){
                    return null;
                }

                command = bytes[0];
                if (command != 0x92)
                {
                    return null;
                    //接続時のみ，0x95が来るが，ここでは無視．

                }
                serialNumber = BitConverter.ToInt64(Util.ReverseBytes(bytes, 1, 8), 0);
                networkAddress = BitConverter.ToInt16(Util.ReverseBytes(bytes, 9, 2), 0);
                digitalEnables = BytesToBool(new byte[2] { bytes[14], bytes[13] });

                analogEnables = BytesToBool(new byte[1] { bytes[15] });

                digitalCnt = CountFlags(digitalEnables);
                analogCnt = CountFlags(analogEnables);
                    
                if (digitalCnt > 0)
                {
                    digitalData = BytesToBool(new byte[2] { bytes[17], bytes[16] });
                }
                else
                {
                    digitalData = null;
                }

                if (analogCnt > 0)
                {
                    analogData = new short[4];//analogCnt
                    int startIndex;
                    if (digitalCnt > 0)
                    {
                        startIndex = 18;
                    }
                    else
                    {
                        startIndex = 16;
                    }


                    for (int i = 0; i < analogData.Length; i++)
                    {
                        if (analogEnables[i])
                        {
                            analogData[i] = BitConverter.ToInt16(Util.ReverseBytes(bytes, startIndex, 2), 0);  //startByte + i * 2
                            startIndex += 2;
                        }
                        else
                        {
                            analogData[i] = 0;
                            //startIndex += 2;
                        }
                    }
                }
                else
                {
                    analogData = null;
                }

                device = new DataDevice(serialNumber, networkAddress, digitalEnables, analogEnables, 
                                            digitalData, analogData);
            }

            catch (Exception ex)
            {
                device = null;
            }

            return device;
           
        }

    }
}
