/*
 * Copyright (c) 2010, Koji Tsukada<tsuka@mobiquitous.com> 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;

using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Collections;
using System.Text.RegularExpressions;
using System.IO;
using System.Threading;

namespace MobiServer.Devices.Xbee
{
    public partial class ControlPanel : MobiServer.ControlBase
    {

        #region Constants/Variables
        Hashtable controlHolder = null;
        ControlDevice targetControl = null;
        int maxScreenHeight = System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Height - 50;
        int maxScreenWidth = System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Width - 50;
        int targetHeight = 0;
        int controlWidthSum = 0;

        public MobiServer.TCPServerControl serverControl;

        const int START_WAIT = 1000;

        const string STR_OPEN_PORT = "|[g{0}J܂D";
        const string STR_CLOSE_PORT = "|[g{0}܂D";
        const string STR_ERROR_DEVICETYPE = "foCX̎ނsłD";

        public enum AddressModes{SerialNumber, NetworkAddress}; //Both
        public AddressModes addressMode = AddressModes.NetworkAddress;
        #endregion

        #region Constructor/Destructor
        public ControlPanel()
        {
            InitializeComponent();
        }

        public ControlPanel(MobiServer.TCPServerControl server, MobiServer.MessageControl message)
            : base(message)
        {
            InitializeComponent();
            serverControl = server;
            controlHolder = new Hashtable();
            comboBoxSettingAddress.SelectedIndex = (int)addressMode;
        }

        private void Panel_Load(object sender, EventArgs e)
        {
            LoadSetting();        

            serverControl.AddReadHandler(new MobiServer.ServerEventHandler(OnServerReadEvent));
            serverControl.SetTitle("TCP Server");
            
            serialPortControl1.serialPort.DataReceived +=new System.IO.Ports.SerialDataReceivedEventHandler(OnDataReceived);
            serialPortControl1.MessageControl = MessageControl; //+= new MobiServer.MessageHandler(MessageControl.OnMessageEvent);

            serialPortControl1.SetTitle("SerialPort");
            serialPortControl1.LoadSetting(); 
            SetTitle(DataServer.Header);
            //initialized = true;
        }


        #endregion

        #region Serial In/Out

        public void CreateXbeeEvent(DataDevice device) 
        {
            XbeeEventArgs e = new MobiServer.Devices.Xbee.XbeeEventArgs(device);
            
            foreach (ControlDevice control in controlHolder.Values)
            {
                if (control.xbeeEventHandler != null)
                {
                    control.xbeeEventHandler.Invoke(this, e);
                }
            }
            
            return;
        }

        void OnDataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            try
            {
                if (e.EventType == System.IO.Ports.SerialData.Chars)
                {
                    if (serialPortControl1.BytesToRead == 0)
                        return;

                    byte b = (byte)serialPortControl1.ReadByte();
                    if (b == DataDevice.START_BYTE)
                    {
                        byte[] bytes = new byte[2];
                        serialPortControl1.ReadBytes(ref bytes, 0);
                        bytes = Util.ReverseBytes(bytes);
                        int byteLength = BitConverter.ToInt16(bytes, 0);

                        bytes = new byte[byteLength + 1];
                        serialPortControl1.ReadBytes(ref bytes, 0);
                        DataDevice device = DataDevice.Parse(bytes);

                        if (device != null)
                        {
                            if (!CheckControlExist(device))
                            {
                                AddControl(new ControlDevice(MessageControl,serverControl, device, addressMode));
                            }
                            CreateXbeeEvent(device);
                        }
                    }

                }
                else if (e.EventType == System.IO.Ports.SerialData.Eof)
                {
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.ToString());
            }
        }

        
        bool CheckControlExist(DataDevice device)
        {
            for (int i = 0; i < controlHolder.Count; i++)
            {
                ControlDevice c = (ControlDevice)controlHolder[i];

                if (c.CheckAddress(device.SerialNumber, device.NetworkAddress))
                {
                    return true;
                }
            }
            return false;
        }

        #endregion

        #region Server In/Out

        public void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {
            try
            {
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
        }

        internal void WriteServerSockets(string[] args)
        {
            serverControl.WriteAllSockets(args);
        }

        #endregion

        #region Add Controls

        public void AddControl(ControlDevice control)
        {
            try
            {
                bool controlExisted = false;
                foreach (ControlDevice c in controlHolder.Values)
                {
                    if (control.CheckAddress(c.Address))
                    {
                        int num = this.Controls.IndexOf(c);
                        this.Controls[num].Enabled = true;
                        this.Refresh();
                        controlExisted = true;
                        break;
                    }
                }

                if (!controlExisted)
                {
                    targetControl = control;
                    controlHolder.Add(controlHolder.Count, control);
                    this.Invoke(new EventHandler(AddControl));
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
            return;


        }

        void AddControl(object sender, EventArgs e)
        {
            flowLayoutPanelDevice.SuspendLayout();
            flowLayoutPanelDevice.Controls.Add(targetControl);

            //Auto-layout based on screen width
            if (controlHolder.Count == 1)
            {
                ResizePanelHeight();
                controlWidthSum = targetControl.Width;
            }
            else
            {
                if (controlWidthSum + targetControl.Width > maxScreenWidth)
                {
                    ResizePanelHeight();

                    if (targetControl.Width > flowLayoutPanelDevice.Width)
                    {
                        ResizePanelWidth(targetControl.Width - flowLayoutPanelDevice.Width);
                    }

                    controlWidthSum = targetControl.Width;
                }
                else
                {
                    if (controlWidthSum + targetControl.Width > flowLayoutPanelDevice.Width)
                    {
                        ResizePanelWidth(targetControl.Width);
                    }
                    controlWidthSum += targetControl.Width;
                }
            }

            flowLayoutPanelDevice.ResumeLayout(true);
        }


        void ResizePanelHeight()
        {
            int targetHeight = targetControl.Height + 10;

            if ((this.Height + this.Location.Y + targetHeight) <= maxScreenHeight)
            {
                flowLayoutPanelDevice.Height += targetHeight;
                this.Height += targetHeight;
            }
            else
            {
                flowLayoutPanelDevice.Height += targetHeight;
                this.Height = maxScreenHeight - this.Location.Y;
            }

        }

        void ResizePanelWidth(int targetWidth)
        {
            if ((this.Width + targetWidth + this.Location.X) <= maxScreenWidth)
            {
                flowLayoutPanelDevice.Width += targetWidth;
                this.Width += targetWidth;
            }
            else
            {
                flowLayoutPanelDevice.Width += targetWidth;
                this.Width = maxScreenWidth - this.Location.X;
            }
        }

        #endregion

        #region Save & Load

        public  void SaveSetting()
        {
            StreamWriter fileStream = null;
            string file = GetSettingFile();
            try
            {
                fileStream = new StreamWriter(file, false);

                fileStream.WriteLine("{0}\t{1}", comboBoxSettingAddress.SelectedIndex, checkBoxPortState.Checked);
            
            }
            catch (Exception ex)
            {
                PrintMessage(ex.Message);
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        public void LoadSetting()
        {
            StreamReader fileStream = null;
            string file = GetSettingFile();
            string line = "";
            string[] args = null;

            try
            {
                if (!File.Exists(file))
                    return;

                fileStream = new StreamReader(file);

                line = fileStream.ReadLine();
                args = line.Split("\t".ToCharArray());
                comboBoxSettingAddress.SelectedIndex = int.Parse(args[0]);
                checkBoxPortState.Checked = bool.Parse(args[1]);
               
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        #endregion

        #region Set Address

        private void comboBoxSettingAddress_SelectedIndexChanged(object sender, EventArgs e)
        {
            addressMode = (AddressModes)comboBoxSettingAddress.SelectedIndex;

            for (int i = 0; i < controlHolder.Count; i++)
            {
                ControlDevice c = (ControlDevice)controlHolder[i];//lineList[j];
                c.SetAddressMode(addressMode);
            }
        }

        private void checkBoxPortState_CheckedChanged(object sender, EventArgs e)
        {
            bool flag = checkBoxPortState.Checked;
            for (int i = 0; i < controlHolder.Count; i++)
            {
                ControlDevice c = (ControlDevice)controlHolder[i];//lineList[j];
                c.SetUpdateStateFlag(flag);
            }
        }
        #endregion
    }
}

